#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# License: GNU General Public License v3

# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2025-03-17
# File  : smh.py


"""
replace cisco ASA ASDM DM_INLINE_NETWORK_ groups with there members in Check Point SmartMove policy file
"""

from argparse import (
    ArgumentParser,
    Namespace,
    RawTextHelpFormatter,
)
from dataclasses import dataclass
from pathlib import Path
from sys import exit as sys_exit

__SMH_VERSION__:str = '0.0.1-20250325'
__SMH_AUTHOR__:str = 'thl-cmk@outlook.com'
__SMH_URL__:str = 'https://thl-cmk.hopto.org/gitlab/bits-and-bytes/SmartMoveHelper'


@dataclass
class Args(Namespace):
    """CLI options for easy access in IDE

    Args:
        Namespace (_type_): ArgumentParser name space
    """
    source_host: str
    fix_fw1: bool


def parse_arguments() -> Args:
    """Argument parser

    Returns:
        arg_Namespace: CLI Arguments
    """
    parser = ArgumentParser(
        prog='smh.py',
        description=(
            'The script replaces ASDM DM_INLINE_NETWORK_ groups with there members in\n'
            'policy import scripts created by Check Point SmartMove\n'

        ),
        formatter_class=RawTextHelpFormatter,
        epilog=f'\nFor more information see: {__SMH_URL__}\n'
                f'Version: {__SMH_VERSION__} | Written by: {__SMH_AUTHOR__}\n'
    )

    parser.add_argument(
        "-s", "--source-host", type=str,
        help='Name of the source host'
    )
    parser.add_argument(
        "-f", "--fix-fw1", action='store_const', const=True,  # default=False,
        help='Replaces "" with "FW1" in policy and objects file',
    )

    return parser.parse_args()


def replace_inline_object(position: str, line:str, group_objects: dict) -> str:
    """
    Replaces DM_INLINE_NETWORK_XXX with the members of the inline group

    Args:
        position (str): position of the inline group to replace (source|destination)
        line (str): line from the policy script where to replace the inline group(s)
        group_objects (dict): dictionary with inline groups name as key and list 
                                of member names as items

    Returns:
        str: policy script line with inline group replaced by group members
    """
    for group_name, group_members in group_objects.items():
        if f'{position} "{group_name}"' in line:
            count: int = 0
            replace_str: str = ''
            for member in group_members:
                replace_str += f' {position}.{count} "{member}" '
                count += 1
            line = line.replace(f'{position} "{group_name}"', replace_str)
    return line


def get_inline_objects(host_name:str) -> dict[str, list[str]]:
    """Create dictionary of inline groups with there members as items

    Args:
        host_name (str): name of the source host. smh expects a file 
                        named "host_name_objects.sh"  in the same directory

    Returns:
        dict: dictionary with inline groups name as key and list of member names as items
    """
    dm_inline_network: str = "cmd='mgmt_cli add group name \"DM_INLINE_NETWORK_"

    inline_objects: dict[str, list[str]] = {}
    object_file = Path(f'{host_name}_objects.sh')

    if object_file.exists():
        raw_objects = object_file.read_text(encoding="UTF-8").splitlines()
        raw_inline_objects: list[str] = [
            line for line in raw_objects if line.startswith(dm_inline_network)
        ]

        for entry in raw_inline_objects:
            entry = entry.split('"')
            name = entry[1]
            members = [
                member for member in entry[2:-3] if not member.strip().startswith("members.")
            ]
            inline_objects[name]=members
        print(f"Number of inline groups: {len(inline_objects)}")
    else:
        print(f"file {object_file.name} not found")

    return inline_objects


def fix_fw1_in_objects(host_name: str) -> None:
    """SmartMove has an issue with service 256/TCP (FW1). It will put only 
    a "" in the policy script instead of "FW1". This function will fix this by replacing 
    each "" witch "FW1". The result is written to "host_name_objects_fixed_fw1.sh".

    Args:
        host_name (str): name of the source host. smh expects a file named "host_name_objects.sh" 
                        in the same directory
    """
    object_file = Path(f'{host_name}_objects.sh')
    if object_file.exists():
        raw_text = object_file.read_text(encoding="UTF-8")
        fixed_objects = Path(f'{host_name}_objects_fixed_fw1.sh')
        fixed_objects.write_text(encoding="UTF-8", data=raw_text.replace('""', '"FW1"'))
        print(f"policy written to: {fixed_objects.name}")


def main():
    """Replace all DM_INLINE_NETWORK groups in the policy file with there members.
    The result is written to "host_name_policy_no_inline.sh"
    """
    args: Args = parse_arguments()
    if not args.source_host:
        print("Usage:")
        print("python convert_inline.py -s <source_host_name> \n\n")
        sys_exit(1)

    inline_objects: dict[str, list[str]] = get_inline_objects(host_name=args.source_host)

    policy_file = Path(f"{args.source_host}_policy.sh")
    policy_no_inline: list[str] = []
    if  policy_file.exists():
        raw_policy: list[str] = policy_file.read_text(encoding="UTF-8").splitlines()
        for raw_line in raw_policy:
            if 'source "DM_INLINE_NETWORK_' in raw_line:
                raw_line = replace_inline_object(
                    position="source", line=raw_line, group_objects=inline_objects
                )

            if 'destination "DM_INLINE_NETWORK_' in raw_line:
                raw_line = replace_inline_object(
                    position="destination", line=raw_line, group_objects=inline_objects
                )

            if args.fix_fw1:
                raw_line = raw_line.replace('""', '"FW1"')

            policy_no_inline.append(raw_line)

        policy_no_inline_file = Path(f"{args.source_host}_policy_no_inline.sh")
        policy_no_inline_file.write_text(encoding="UTF-8", data="\n".join(policy_no_inline))
        print(f"policy written to: {policy_no_inline_file.name}")

        if args.fix_fw1:
            fix_fw1_in_objects(args.source_host)
    else:
        print(f"file {policy_file.name} not found")

if __name__ == '__main__':
    main()
